@@ -12,6 +12,8 @@
 #' ---
 #'
 #+ include=FALSE
+here::i_am("inst/sbc/sbc_report.R")
+library(here)
 library(knitr)
 library(tools)
 library(assertthat)
@@ -20,7 +22,7 @@ library(tidyr)
 library(broom)
 library(ggplot2)
 theme_set(theme_bw())
-source("sbc_tools.R")
+source(here("inst", "sbc", "sbc_tools.R"))
 
 knitr::opts_chunk$set(
   fig.width = 1.62*4,
@@ -34,11 +36,11 @@ knitr::opts_chunk$set(
 #'
 #' The calibration data presented here has been generated at and with
 #' the `OncoBayes` git version as:
-cat(readLines("calibration.md5"), sep="\n")
+cat(readLines(here("inst", "sbc", "calibration.md5")), sep="\n")
 #'
 #' The MD5 hash of the calibration data file presented here must match
 #' the above listed MD5:
-md5sum("calibration.rds")
+md5sum(here("inst", "sbc", "calibration.rds"))
 #'
 #' # Introduction
 #'
@@ -84,7 +86,7 @@ md5sum("calibration.rds")
 #' bin.
 #'
 
-calibration <- readRDS("calibration.rds")
+calibration <- readRDS(here("inst", "sbc", "calibration.rds"))
 
 
 include_plots <- TRUE
@@ -104,27 +106,27 @@ plot_binned <- function(cal_df) {
 
     if(!include_plots)
         return(pl)
-    
+
     if(!all(cal_df$count == 0)) {
-        
+
         S <- calibration$S
         B <- calibration$B
-        
+
         c95 <- qbinom(c(0.025, 0.5, 0.975), S, 1 / B)
-        
+
         dd <- cal_df %>%
             arrange(param, bin) %>%
                 group_by(param) %>%
                     mutate(ecdf = cumsum(count) / S, ecdf_ref = (bin + 1) / B) %>%
                         filter(!all(ecdf == 0))
-        
+
         nparam <- length(unique(dd$param))
         if(unique(dd$partype) %in% c("mu_eta", "tau_eta")){
             nc <- nparam
         } else{
             nc <- 2
         }
-        
+
         nr <- max(1, ceiling(nparam / nc))
 
         pl <- list()
@@ -164,6 +166,52 @@ pl_split <- lapply(cal_split, function(cal_df) plot_binned(cal_df))
 #'
 kable(calibration$sampler_diagnostics, digits=3)
 
+
+#'
+#+ include=include_plots, eval=include_plots, fig.width=8,fig.height=6
+calibration$raw %>%
+    select(starts_with("min_"), "max_Rhat", starts_with("lp_"), "data_scenario") %>%
+    pivot_longer(!data_scenario, names_to="metric") %>%
+    ggplot(aes(value)) +
+    facet_grid(data_scenario~metric, scales="free_x") +
+    geom_histogram(bins=60) + xlab(NULL) +
+    scale_x_log10() +
+    theme(axis.text.x = element_text(angle = 60, vjust = 1, hjust=1)) +
+    ggtitle("Sampler diagnostics")
+
+#'
+#' Large Rhat is defined as exceeding $1.1$.
+#'
+#' ## Sampler Adaptation & Performance Overview
+#'
+#'
+sampler_performance <- calibration$raw %>%
+    select("time.running", starts_with("lp_"), "stepsize", "accept_stat", "data_scenario") %>%
+    mutate(lp_ess_bulk_speed=lp_ess_bulk / time.running, lp_ess_tail_speed=lp_ess_tail / time.running) %>%
+    select("data_scenario", "stepsize", "accept_stat", ends_with("_speed"))
+
+sampler_performance %>% group_by(data_scenario) %>%
+    summarize(across(where(is.numeric), list(mean=mean, sd=sd)), N=n()) %>%
+    relocate(data_scenario, N) %>%
+    kable(digits=3)
+
+#'
+#' ESS speed is in units of ESS per second.
+#'
+
+#+ include=include_plots, eval=include_plots, fig.width=8,fig.height=6
+sampler_performance %>%
+    pivot_longer(!data_scenario, names_to="metric") %>%
+    mutate(metric=factor(metric, c("accept_stat", "stepsize", "lp_ess_bulk_speed", "lp_ess_tail_speed"))) %>%
+    ggplot(aes(value)) +
+    facet_grid(data_scenario~metric, scales="free_x") +
+    geom_histogram(bins=60) + xlab(NULL) +
+    scale_x_log10() +
+    theme(axis.text.x = element_text(angle = 60, vjust = 1, hjust=1)) +
+    ggtitle("Sampler adaptation & performance")
+
+#'
+#'
 chisq  <- bins_all %>%
   arrange(data_scenario, partype, param, bin) %>%
   group_by(data_scenario, partype, param) %>%
@@ -173,10 +221,6 @@ chisq  <- bins_all %>%
   rename(df = parameter) %>%
   ungroup()
 
-#'
-#' Large Rhat is defined as exceeding $1.2$.
-#' 
-
 #'
 #' ## $\chi^2$ Statistic, Model 1: Single-agent logistic regression
 #'
